"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getParentFunctionOrStatement = getParentFunctionOrStatement;
exports.markNodeAsTracked = markNodeAsTracked;
exports.isNodeTracked = isNodeTracked;
exports.isNodeUsed = isNodeUsed;
exports.markNodeAsUsed = markNodeAsUsed;
exports.isNodeBindingUsed = isNodeBindingUsed;
exports.getSideInDeclaration = getSideInDeclaration;
exports.getSideInObjectProperty = getSideInObjectProperty;
exports.getSideInAssignmentExpression = getSideInAssignmentExpression;
exports.getSideInImportSpecifier = getSideInImportSpecifier;
exports.SYM_IDENTIFIER_TRACKED = exports.SYM_IDENTIFIER_USED = void 0;
const SYM_IDENTIFIER_USED = Symbol('IDENTIFIER_USED');
exports.SYM_IDENTIFIER_USED = SYM_IDENTIFIER_USED;
const SYM_IDENTIFIER_TRACKED = Symbol('IDENTIFIER_TRACKED');
exports.SYM_IDENTIFIER_TRACKED = SYM_IDENTIFIER_TRACKED;

function getParentFunctionOrStatement(path) {
  do {
    if (!path.parentPath || Array.isArray(path.container) && path.isStatement() || path.isFunction()) {
      break;
    } else {
      path = path.parentPath;
    }
  } while (path);

  if (path && (path.isProgram() || path.isFile())) {
    return null;
  }

  return path;
}

function markNodeAsTracked(node) {
  node[SYM_IDENTIFIER_TRACKED] = true;
  node[SYM_IDENTIFIER_USED] = false;
}

function isNodeTracked(node) {
  return !!node[SYM_IDENTIFIER_TRACKED];
}

function isNodeUsed(node) {
  return !!node[SYM_IDENTIFIER_USED];
}

function markNodeAsUsed(path) {
  const binding = path.scope.getBindingIdentifier(path.node.name);

  if (binding) {
    if (!isNodeTracked(binding)) {
      return;
    }

    binding[SYM_IDENTIFIER_USED] = true;
  }
}

function isNodeBindingUsed(path) {
  const binding = path.scope.getBindingIdentifier(path.node.name);

  if (binding) {
    if (!isNodeTracked(binding)) {
      return true;
    }

    return binding[SYM_IDENTIFIER_USED];
  }

  return true;
}

function getSideInDeclaration(child, parent) {
  let currentItem = child;
  const {
    declarations
  } = parent.node;
  const length = declarations.length;

  do {
    for (let i = 0; i < length; i++) {
      if (declarations[i].id === currentItem.node) {
        return 'left';
      }

      if (declarations[i].init === currentItem.node) {
        return 'right';
      }
    }

    currentItem = currentItem.parentPath;
  } while (currentItem);

  return null;
}

function getSideInObjectProperty(child, parent) {
  let currentItem = child;

  do {
    if (currentItem.node === parent.node.key) {
      return 'left';
    }

    if (currentItem.node === parent.node.value) {
      return 'right';
    }

    currentItem = currentItem.parentPath;
  } while (currentItem);

  return null;
}

function getSideInAssignmentExpression(child, parent) {
  let currentItem = child;

  do {
    if (currentItem.node === parent.node.left) {
      return 'left';
    }

    if (currentItem.node === parent.node.right) {
      return 'right';
    }

    currentItem = currentItem.parentPath;
  } while (currentItem);

  return null;
}

function getSideInImportSpecifier(child) {
  let currentItem = child;

  do {
    const parentPath = currentItem.parentPath;

    if (parentPath.node) {
      if (currentItem.node === parentPath.node.imported) {
        return 'left';
      }

      if (currentItem.node === parentPath.node.local) {
        return 'right';
      }
    } else break;
  } while (currentItem);

  return null;
}