"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _common = require("./common");

var _default = {
  Identifier(path, {
    t
  }) {
    const parentPath = path.parentPath;

    if (t.isMemberExpression(parentPath)) {
      if (path.node === parentPath.node.object) {
        (0, _common.markNodeAsUsed)(path);
      }

      return;
    }

    const statementParent = (0, _common.getParentFunctionOrStatement)(path);

    if (t.isImportDeclaration(statementParent)) {
      // Handle recursive destructuring here
      if (t.isImportDefaultSpecifier(parentPath)) {
        (0, _common.markNodeAsTracked)(path.node);
      } else {
        const sideIms = (0, _common.getSideInImportSpecifier)(path);

        if (sideIms === 'right') {
          (0, _common.markNodeAsTracked)(path.node);
        }
      }

      return;
    }

    if (t.isExpressionStatement(statementParent)) {
      const expressionPath = statementParent.get('expression');

      if (t.isCallExpression(expressionPath)) {
        (0, _common.markNodeAsUsed)(path);
      } else if (t.isAssignmentExpression(expressionPath)) {
        const sideAse = (0, _common.getSideInAssignmentExpression)(path, expressionPath);

        if (sideAse === 'right') {
          (0, _common.markNodeAsUsed)(path);
        }
      }

      return;
    }

    if (t.isVariableDeclaration(statementParent)) {
      const sideDecl = (0, _common.getSideInDeclaration)(path, statementParent);

      if (sideDecl === 'left') {
        if (t.isObjectProperty(parentPath)) {
          const objPropSide = (0, _common.getSideInObjectProperty)(path, parentPath);

          if (objPropSide === 'right') {
            // is a decl
            (0, _common.markNodeAsTracked)(path.node);
          }
        } else if (t.isVariableDeclarator(parentPath)) {
          (0, _common.markNodeAsTracked)(path.node);
        } else {//
        }

        return;
      }

      if (sideDecl === 'right') {
        (0, _common.markNodeAsUsed)(path);
        return;
      }

      return;
    }

    if (t.isFunction(statementParent)) {
      // Identifier is a parameter or in function body
      if (t.isFunction(parentPath)) {
        // We have a simple param identifier
        (0, _common.markNodeAsTracked)(path.node);
      } else if (t.isObjectProperty(parentPath)) {
        const objPropSide = (0, _common.getSideInObjectProperty)(path, parentPath);

        if (objPropSide === 'right') {
          (0, _common.markNodeAsTracked)(path.node);
        }
      } else if (t.isAssignmentExpression(parentPath)) {
        const sideAse = (0, _common.getSideInAssignmentExpression)(path, parentPath);

        if (sideAse === 'right') {
          (0, _common.markNodeAsUsed)(path);
        }
      } else {
        (0, _common.markNodeAsUsed)(path);
      }

      return;
    }
  }

};
exports.default = _default;