"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _common = require("./common");

function removeObjectPropertyDecl({
  path,
  parentPath,
  statementParent
}) {
  const parentObjPropPath = parentPath;
  const objPropSide = (0, _common.getSideInObjectProperty)(path, parentObjPropPath);

  if (objPropSide === 'right') {
    if (!(0, _common.isNodeUsed)(path.node)) {
      parentPath.remove();
      let currentPath = parentObjPropPath.parentPath;

      for (;;) {
        const currentNode = currentPath.node;

        if (currentNode.properties.length === 0) {
          if (currentPath.parentPath.isObjectProperty()) {
            const grandpaObject = currentPath.parentPath.parentPath.node;
            grandpaObject.properties = grandpaObject.properties.filter(item => item !== currentPath.parentPath.node);
            currentPath = currentPath.parentPath.parentPath;
          } else {
            if (currentPath.parentPath.isVariableDeclarator()) {
              // No properties and parent path is root. remove the entire thing.
              statementParent.remove();
            } else if (currentPath.parentPath.isFunction()) {
              // ObjectPattern is as a parameter in a function
              statementParent.remove();
            }

            break;
          }
        } else {
          break;
        }
      }
    }
  }
}

var _default = {
  Identifier(path, {
    t
  }) {
    const parentPath = path.parentPath;

    if (t.isMemberExpression(parentPath)) {
      return;
    }

    const statementParent = (0, _common.getParentFunctionOrStatement)(path);

    if (!statementParent) {
      return;
    }

    if (t.isImportDeclaration(statementParent)) {
      if (t.isImportDefaultSpecifier(parentPath)) {
        if (!(0, _common.isNodeUsed)(path.node)) {
          statementParent.remove();
        }

        return;
      } else {
        const sideIms = (0, _common.getSideInImportSpecifier)(path);

        if (sideIms === 'right') {
          if (!(0, _common.isNodeUsed)(path.node)) {
            parentPath.remove();

            if (!statementParent.node.specifiers.length) {
              statementParent.remove();
            }
          }
        }
      }

      return;
    }

    if (t.isExpressionStatement(statementParent)) {
      const expressionPath = statementParent.get('expression');

      if (t.isCallExpression(expressionPath)) {
        // - mark as used
        return;
      } else if (t.isAssignmentExpression(expressionPath)) {//
      }

      return;
    }

    if (t.isVariableDeclaration(statementParent)) {
      const sideDecl = (0, _common.getSideInDeclaration)(path, statementParent);

      if (sideDecl === 'left') {
        if (t.isObjectProperty(parentPath)) {
          removeObjectPropertyDecl({
            path,
            parentPath,
            statementParent
          });
        } else if (t.isVariableDeclarator(parentPath)) {
          if (!(0, _common.isNodeUsed)(path.node)) {
            statementParent.remove();
          }
        } else {}
      } // Ignore right side here, remove the entire statement if left is unused


      return;
    }

    if (t.isFunction(statementParent)) {
      // Identifier is a parameter or in function body
      if (t.isFunction(parentPath)) {
        // We have a simple param identifier
        // Only remove if last param
        const parentNode = parentPath.node;

        if (parentNode.params[parentNode.params.length - 1] === path.node && !(0, _common.isNodeUsed)(path.node)) {
          path.remove();
        }
      }

      if (t.isObjectProperty(parentPath)) {
        const paramParent = parentPath.find(path => // We found an object pattern and it's a direct discendent of this function aka the param.
        path.isObjectPattern() && path.parentPath.node === statementParent.node || // We reached the end of the function decl
        path.node === statementParent.node);

        if (paramParent && paramParent.isObjectPattern()) {
          removeObjectPropertyDecl({
            path,
            parentPath,
            statementParent: paramParent
          });
        }
      }

      return;
    }
  }

};
exports.default = _default;